@@ -34,6 +34,7 @@ and then returned to the caller.
 import collections
 import inspect
 import itertools
+import urllib.parse
 
 import jsonpatch
 import operator
@@ -41,7 +42,6 @@ from keystoneauth1 import adapter
 from keystoneauth1 import discover
 import munch
 from requests import structures
-import six
 
 from openstack import _log
 from openstack import exceptions
@@ -82,7 +82,7 @@ def _convert_type(value, data_type, list_type=None):
         return data_type(value)
 
 
-class _BaseComponent(object):
+class _BaseComponent:
 
     # The name this component is being tracked as in the Resource
     key = None
@@ -257,7 +257,7 @@ class _ComponentManager(collections.MutableMapping):
             self._dirty = set()
 
 
-class _Request(object):
+class _Request:
     """Prepared components that go into a KSA request"""
 
     def __init__(self, url, body, headers):
@@ -266,7 +266,7 @@ class _Request(object):
         self.headers = headers
 
 
-class QueryParameters(object):
+class QueryParameters:
 
     def __init__(self, *names, **mappings):
         """Create a dict of accepted query parameters
@@ -1094,7 +1094,7 @@ class Resource(dict):
             uri = utils.urljoin(uri, self.id)
 
         if params:
-            query_params = six.moves.urllib.parse.urlencode(params)
+            query_params = urllib.parse.urlencode(params)
             uri += '?' + query_params
 
         return _Request(uri, body, headers)
@@ -1117,6 +1117,13 @@ class Resource(dict):
                 if self.resource_key and self.resource_key in body:
                     body = body[self.resource_key]
 
+                # Do not allow keys called "self" through. Glance chose
+                # to name a key "self", so we need to pop it out because
+                # we can't send it through cls.existing and into the
+                # Resource initializer. "self" is already the first
+                # argument and is practically a reserved word.
+                body.pop("self", None)
+
                 body_attrs = self._consume_body_attrs(body)
 
                 if self._store_unknown_attrs_as_properties:
@@ -1776,11 +1783,10 @@ class Resource(dict):
         # This prevents duplication of query parameters that with large
         # number of pages result in HTTP 414 error eventually.
         if next_link:
-            parts = six.moves.urllib.parse.urlparse(next_link)
-            query_params = six.moves.urllib.parse.parse_qs(parts.query)
+            parts = urllib.parse.urlparse(next_link)
+            query_params = urllib.parse.parse_qs(parts.query)
             params.update(query_params)
-            next_link = six.moves.urllib.parse.urljoin(next_link,
-                                                       parts.path)
+            next_link = urllib.parse.urljoin(next_link, parts.path)
 
         # If we still have no link, and limit was given and is non-zero,
         # and the number of records yielded equals the limit, then the user
@@ -1863,7 +1869,7 @@ class Resource(dict):
             "No %s found for %s" % (cls.__name__, name_or_id))
 
 
-class TagMixin(object):
+class TagMixin:
 
     _tag_query_parameters = {
         'tags': 'tags',
